# General

DEVICE_CPU = "cpu"

DEVICE_GPU = "gpu"

DEVICE_MPS = "mps"

EXPECTED_SAMPLE_RATE = 16000

EXPECTED_FORMAT = "wav"

EXPECTED_SAMPLE_MINUTE = EXPECTED_SAMPLE_RATE * 60

# Parakeet

PARAKEET_MODEL_REF = "nvidia/parakeet-tdt-0.6b-v3"

PARAKEET_INFERENCE_MAX_DURATION_MIN = (
    20  # actually 24, but we want to give ourselves some room
)

PARAKEET_INFERENCE_MAX_DURATION_KHZ = (
    PARAKEET_INFERENCE_MAX_DURATION_MIN * EXPECTED_SAMPLE_MINUTE
)

PARAKEET = "parakeet"

# Whisper

WHISPER_CPP = "whisper-cpp"
